# Git Workflow Conventions

## Branch Management

### Main Branch
- The `main` branch contains the production-ready code
- Direct commits to `main` are not allowed
- All changes must come through reviewed merge requests

### Feature Branches
- Create a new branch for each feature, bug fix, or improvement
- Branch naming convention:
  ```
  <type>/<description>
  ```
  where `type` is one of:
  - `feature`: New functionality
  - `bugfix`: Bug fixes
  - `hotfix`: Critical fixes for production
  - `refactor`: Code improvements without feature changes
  - `docs`: Documentation updates

Examples:
- `feature/user-authentication`
- `bugfix/login-validation`
- `refactor/optimize-queries`

## Commit Messages
All commit messages should follow this format:
```
<type>[optional scope]: <subject>

[optional body]
[optional footer]
```

### Types
- `feat`: New feature
- `fix`: Bug fix
- `docs`: Documentation changes
- `style`: Formatting, missing semicolons, etc.
- `refactor`: Code restructuring
- `test`: Adding missing tests
- `chore`: Maintenance tasks

### Scope
The scope is optional and provides additional context about the section of the codebase being modified. Common scopes include:
- Component names
- Feature areas
- Module names
- Specific services

Examples:
```
feat(auth): add password reset functionality

- Implement password reset email sending
- Add reset token verification
- Create new password form
```

```
fix(session): correct user timeout duration

The session was timing out after 15 minutes instead of the
intended 30 minutes. Updated the session configuration.

Fixes: #123
```

```
refactor(api): simplify error handling in user routes

Consolidated duplicate error handling code and improved
error messages for better debugging.
```

### Guidelines
- Subject line should be no longer than 50 characters
- Use imperative mood ("Add feature" not "Added feature")
- Do not end the subject line with a period
- Separate subject from body with a blank line
- Use the body to explain what and why vs. how
- Scope should be in lowercase
- Multiple scopes can be combined with slash: `(api/auth)`

## Merge Request Process

### Creating a Merge Request
1. Push your feature branch to GitLab
2. Create a merge request targeting the `main` branch
3. Fill out the merge request template
4. Assign reviewers

### Requirements for Merging
- At least one approval from a team member
- All discussions must be resolved
- CI pipeline must pass
- Branch must be up to date with `main`

### Code Review Guidelines
- Review for:
  - Code quality and style
  - Potential bugs
  - Test coverage
  - Documentation
  - Security concerns
- Provide constructive feedback
- Respond to review comments promptly

### After Merge
- Delete the feature branch
- Update your local repository
- Close related issues

## Best Practices
- Keep branches short-lived (1-2 weeks maximum)
- Regularly sync your feature branch with `main`
- Write clear commit messages
- Break large changes into smaller, logical chunks
- Always write tests for new features
- Update documentation as needed

## Getting Started
```bash
# Create a new feature branch
git checkout main
git pull
git checkout -b feature/new-feature

# Make changes and commit
git add .
git commit -m "feat(user): add profile editing"

# Push branch and create merge request
git push -u origin feature/new-feature
```
