'use client'

import {
  Pagination,
  PaginationContent,
  PaginationEllipsis,
  PaginationItem,
  PaginationLink,
  PaginationNext,
  PaginationPrevious,
} from '@/components/ui/pagination'
import { usePathname, useRouter } from 'next/navigation'
import React from 'react'

interface PaginationUIProps {
  currentPage: number
  totalPages: number
  totalDocs: number
}

export default function PaginationUI({
  currentPage,
  totalPages,
  totalDocs,
}: PaginationUIProps): React.ReactNode {
  const router = useRouter()
  const pathname = usePathname()

  // Generate page numbers to display
  const getPageNumbers = (): number[] => {
    const pageNumbers: number[] = []

    pageNumbers.push(1)

    // Pages around current page
    for (let i = Math.max(2, currentPage - 1); i <= Math.min(totalPages - 1, currentPage + 1); i++) {
      pageNumbers.push(i)
    }

    // Always show last page if we have more than one page
    if (totalPages > 1) {
      pageNumbers.push(totalPages)
    }

    return [...new Set(pageNumbers)].sort((a, b) => a - b)
  }

  const pageNumbers = getPageNumbers()

  // Navigate to a specific page
  const handlePageChange = (page: number): void => {
    router.push(`${pathname}?page=${page}`)
  }

  if (totalPages <= 1) {
    return null
  }

  return (
    <div className="flex flex-col items-center gap-2">
      <p className="text-sm text-gray-500">
        Showing
        {' '}
        {Math.ceil((currentPage - 1) * (totalDocs / totalPages) + 1)}
        {' '}
        to
        {' '}
        {Math.ceil(Math.min(currentPage * (totalDocs / totalPages), totalDocs))}
        {' '}
        of
        {' '}
        {totalDocs}
        {' '}
        posts
      </p>

      <Pagination>
        <PaginationContent>
          {/* Previous button */}
          {currentPage > 1 && (
            <PaginationItem>
              <PaginationPrevious
                onClick={() => handlePageChange(currentPage - 1)}
                className="cursor-pointer"
              />
            </PaginationItem>
          )}

          {/* Page numbers */}
          {pageNumbers.map((pageNumber, index) => {
            const showEllipsisBefore = index > 0 && pageNumber > pageNumbers[index - 1] + 1

            return (
              <React.Fragment key={pageNumber}>
                {showEllipsisBefore && (
                  <PaginationItem>
                    <PaginationEllipsis />
                  </PaginationItem>
                )}

                <PaginationItem>
                  <PaginationLink
                    isActive={currentPage === pageNumber}
                    onClick={() => handlePageChange(pageNumber)}
                    className="cursor-pointer"
                  >
                    {pageNumber}
                  </PaginationLink>
                </PaginationItem>
              </React.Fragment>
            )
          })}

          {/* Next button */}
          {currentPage < totalPages && (
            <PaginationItem>
              <PaginationNext
                onClick={() => handlePageChange(currentPage + 1)}
                className="cursor-pointer"
              />
            </PaginationItem>
          )}
        </PaginationContent>
      </Pagination>
    </div>
  )
}
