import type { CalendarEvent, DaySchedule } from '../../types'

import { Dialog, DialogContent, DialogFooter, DialogHeader, DialogTitle } from '@/components/ui/dialog'
import { Separator } from '@/components/ui/separator'
import { Description } from '@radix-ui/react-dialog'
import { Calendar, Clock, MapPin } from 'lucide-react'
import React, { useCallback, useMemo, useState } from 'react'
import { CalendarCard } from '../CalendarCard'
import { CalendarHeader } from '../CalendarHeader'

interface LoadedStateProps {
  events: CalendarEvent[]
  weekRange: string
  onNavigateWeek: (direction: 'next' | 'previous') => void
  startDate: Date
}

// Utility functions
function getWeekDates(startDate: Date): Date[] {
  const start = new Date(startDate)
  start.setHours(0, 0, 0, 0)

  return Array.from({ length: 7 }, (_, i) => {
    const day = new Date(start)
    day.setDate(start.getDate() + i)
    return day
  })
}

function formatDayName(date: Date): string {
  const days = ['SO', 'MO', 'DI', 'MI', 'DO', 'FR', 'SA']
  return days[date.getDay()]
}

function isSameDate(date1: Date, date2: Date): boolean {
  return date1.getDate() === date2.getDate()
    && date1.getMonth() === date2.getMonth()
    && date1.getFullYear() === date2.getFullYear()
}

function formatTimeRange(start: Date, end: Date): string {
  const formatTime = (date: Date) => {
    return date.toLocaleTimeString('de-DE', {
      hour: '2-digit',
      minute: '2-digit',
      hour12: false,
    })
  }
  return `${formatTime(start)}-${formatTime(end)}`
}

function formatDate(date: Date): string {
  return date.toLocaleDateString('de-DE', {
    weekday: 'long',
    year: 'numeric',
    month: 'long',
    day: 'numeric',
  })
}

// Event Detail Modal Component
interface EventDetailModalProps {
  event: CalendarEvent | null
  isOpen: boolean
  onClose: () => void
}

function EventDetailModal({ event, isOpen, onClose }: EventDetailModalProps) {
  if (!event)
    return null

  return (
    <Dialog open={isOpen} onOpenChange={open => !open && onClose()}>
      <DialogContent aria-describedby="title" className="sm:max-w-md">
        <DialogHeader>
          <DialogTitle>{event.title}</DialogTitle>
          <Description />
        </DialogHeader>

        <Separator />

        <div className="space-y-4 py-4">
          <div className="flex items-center gap-2">
            <Calendar className="w-5 h-5 text-blue-500 dark:text-blue-400" />
            <span className="text-gray-600 dark:text-gray-100">{formatDate(event.start)}</span>
          </div>

          <div className="flex items-center gap-2">
            <Clock className="w-5 h-5 text-blue-500 dark:text-blue-400" />
            <span className="text-gray-600 dark:text-gray-100">{formatTimeRange(event.start, event.end)}</span>
          </div>

          {event.location && (
            <div className="flex items-center gap-2">
              <MapPin className="w-5 h-5 text-blue-500 dark:text-blue-400" />
              <span className="text-gray-600 dark:text-gray-100">{event.location}</span>
            </div>
          )}
        </div>

        <DialogFooter>
          {event.description && (
            <p className="text-gray-600 dark:text-gray-300">{event.description}</p>
          )}
        </DialogFooter>
      </DialogContent>
    </Dialog>
  )
}

// Components
const EventCard: React.FC<{ event: CalendarEvent, onClick: () => void }> = ({ event, onClick }) => (
  <div
    className="bg-blue-500 dark:bg-blue-400 p-4 rounded-lg cursor-pointer hover:bg-blue-600 dark:hover:bg-blue-500 transition-colors"
    onClick={onClick}
  >
    <h3 className="text-gray-100 dark:text-gray-900 font-medium mb-1">{event.title}</h3>
    <p className="text-gray-100 dark:text-gray-900">{formatTimeRange(event.start, event.end)}</p>
  </div>
)

const DayColumn: React.FC<{ day: string, date: string, isToday?: boolean }> = ({
  day,
  date,
  isToday = false,
}) => (
  <div className="w-16 flex flex-col">
    <span className="text-gray-800 dark:text-gray-400 text-sm mb-1">{day}</span>
    <div
      className={`text-3xl ${
        isToday
          ? 'bg-blue-700 text-gray-100 dark:bg-blue-300 dark:text-gray-900 rounded-full w-12 h-12 flex items-center justify-center'
          : 'text-gray-700 dark:text-gray-400'
      }`}
    >
      {date}
    </div>
  </div>
)

const DayRow: React.FC<DaySchedule & { onEventClick: (event: CalendarEvent) => void }>
  = ({ day, date, isToday, events, onEventClick }) => (
    <div className="flex items-start gap-4">
      <DayColumn day={day} date={date} isToday={isToday} />
      <div className="flex-1 space-y-2">
        {events.map(event => (
          <EventCard
            key={event.id}
            event={event}
            onClick={() => onEventClick(event)}
          />
        ))}
      </div>
    </div>
  )

export const LoadedState: React.FC<LoadedStateProps> = ({ events, weekRange, onNavigateWeek, startDate }) => {
  const [selectedEvent, setSelectedEvent] = useState<CalendarEvent | null>(null)
  const [isModalOpen, setIsModalOpen] = useState(false)

  // Memoize week dates and schedule to prevent unnecessary recalculations
  const weekDates = useMemo(() => getWeekDates(startDate), [startDate])
  const today = useMemo(() => new Date(), [])

  const schedule = useMemo(() => {
    return weekDates
      .map((date) => {
        const dayEvents = events.filter(event => isSameDate(event.start, date))

        return {
          day: formatDayName(date),
          date: date.getDate().toString(),
          fullDate: date,
          isToday: isSameDate(date, today),
          events: dayEvents,
        }
      })
      .filter(day => day.events.length > 0)
  }, [weekDates, events, today])

  const handleEventClick = (event: CalendarEvent) => {
    setSelectedEvent(event)
    setIsModalOpen(true)
  }

  const handleNextWeek = useCallback(() => {
    onNavigateWeek('next')
  }, [onNavigateWeek])

  const handlePreviousWeek = useCallback(() => {
    onNavigateWeek('previous')
  }, [onNavigateWeek])

  return (
    <CalendarCard>
      <CalendarHeader
        formattedTimeRange={weekRange}
        onPrevious={handlePreviousWeek}
        onNext={handleNextWeek}
      />
      <div className="space-y-4">
        {schedule.map(daySchedule => (
          <DayRow
            key={daySchedule.fullDate.toISOString()}
            {...daySchedule}
            onEventClick={handleEventClick}
          />
        ))}
      </div>

      <EventDetailModal
        event={selectedEvent}
        isOpen={isModalOpen}
        onClose={() => setIsModalOpen(false)}
      />
    </CalendarCard>
  )
}
