/* eslint-disable node/prefer-global/process */
import type { CalendarEvent, CalendarEventsHook, WeekRange } from '../types'
import { useCallback, useEffect, useState } from 'react'

function getWeekBounds(weekOffset: number = 0) {
  const currentDate = new Date()
  currentDate.setHours(0, 0, 0, 0)

  // Get Monday of current week
  const currentDay = currentDate.getDay()
  const diff = currentDay === 0 ? -6 : 1 - currentDay
  const start = new Date(currentDate)
  start.setDate(currentDate.getDate() + diff + (weekOffset * 7))

  // Get end of week (Sunday)
  const end = new Date(start)
  end.setDate(end.getDate() + 6)
  end.setHours(23, 59, 59, 999)

  const formatted = formatDateRange(start, end)

  return { start, end, formatted }
}

function formatDateRange(start: Date, end: Date): string {
  // If dates are in the same month and year
  if (start.getMonth() === end.getMonth() && start.getFullYear() === end.getFullYear()) {
    return `${start.getDate()}. - ${end.getDate()}. ${start.toLocaleString('de-DE', { month: 'long' })} ${start.getFullYear()}`
  }

  // If dates are in different months but same year
  if (start.getFullYear() === end.getFullYear()) {
    return `${start.getDate()}. ${start.toLocaleString('de-DE', { month: 'short' })} - ${end.getDate()}. ${end.toLocaleString('de-DE', { month: 'short' })} ${start.getFullYear()}`
  }
  // If dates are in different years
  return `${start.getDate()}. ${start.toLocaleString('de-DE', { month: 'short' })} ${start.getFullYear()} - ${end.getDate()}. ${end.toLocaleString('de-DE', { month: 'short' })} ${end.getFullYear()}`
}

export function useCalendarEvents(): CalendarEventsHook {
  const [events, setEvents] = useState<CalendarEvent[]>([])
  const [isLoading, setIsLoading] = useState<boolean>(true)
  const [error, setError] = useState<string | null>(null)
  const [weekOffset, setWeekOffset] = useState<number>(0)
  const [weekRange, setWeekRange] = useState<WeekRange>(getWeekBounds(0))

  const fetchEvents = async (offset: number): Promise<void> => {
    try {
      setIsLoading(true)
      setError(null)

      const newWeekRange = getWeekBounds(offset)
      setWeekRange(newWeekRange)

      const startDate = newWeekRange.start.toISOString()
      const endDate = newWeekRange.end.toISOString()

      const response = await fetch(
        `${process.env.NEXT_PUBLIC_PAYLOAD_API_URL}/api/event?where[and][0][start][greater_than_equal]=${startDate}&where[and][1][start][less_than]=${endDate}&sort=begin_date`,
        {
          method: 'GET',
          headers: {
            'Content-Type': 'application/json',
          },
          credentials: 'include',
        },
      )

      if (!response.ok) {
        throw new Error('Failed to fetch events')
      }

      const data = await response.json()
      const fetchedEvents: CalendarEvent[] = data.docs.map((event: any) => ({
        ...event,
        start: new Date(event.start),
        end: new Date(event.end),
      }))

      setEvents(fetchedEvents)
    }
    catch (err) {
      setError(err instanceof Error ? err.message : 'An unknown error occured')
    }
    finally {
      setIsLoading(false)
    }
  }

  const navigateWeek = useCallback((direction: 'next' | 'previous') => {
    const newOffset = direction === 'next' ? weekOffset + 1 : weekOffset - 1
    setWeekOffset(newOffset)
    fetchEvents(newOffset)
  }, [weekOffset])

  useEffect(() => {
    fetchEvents(weekOffset)
  }, [weekOffset])

  return {
    events,
    isLoading,
    weekRange,
    error,
    refetch: () => fetchEvents(weekOffset),
    navigateWeek,
  }
}
