import type { PayloadNewsArticle } from './types'
import { Card, CardContent, CardDescription, CardFooter, CardHeader, CardTitle } from '@/components/ui/card'
import { ScrollArea, ScrollBar } from '@/components/ui/scroll-area'
import config from '@payload-config'
import { format } from 'date-fns'
import { BookOpen, Calendar, ChevronRight, Newspaper } from 'lucide-react'
import Image from 'next/image'
import Link from 'next/link'
import { getPayload } from 'payload'
import React from 'react'
import Tag from '../Tag'

export default async function NewsWidget() {
  const payload = await getPayload({ config })

  const result = await payload.find({
    collection: 'news-articles',
    sort: '-publishedDate',
    depth: 1,
  })

  const articles = result.docs as unknown as PayloadNewsArticle[]

  return (
    <div className="w-full max-w-full py-6">
      <div className="flex items-center justify-between mb-4">
        <h2 className="text-2xl font-bold text-gray-800 dark:text-white px-4 md:px-0">Latest News</h2>
        <Link
          href="/news"
          className="flex items-center text-sm text-gray-600 hover:text-red-600 dark:text-neutral-300 dark:hover:text-red-400 transition-colors px-4 md:px-0"
        >
          View all news
          <ChevronRight className="ml-1 h-4 w-4" />
        </Link>
      </div>

      <ScrollArea className="w-full whitespace-nowrap">
        <div className="flex space-x-4 pb-4 px-4 md:px-0">
          {articles.length > 0
            ? (
                articles.map(article => (
                  <Card
                    key={article.id}
                    className="w-[350px] flex-shrink-0 bg-white dark:bg-neutral-800 border-gray-200 dark:border-neutral-700 hover:border-red-300 dark:hover:border-red-800 transition-colors"
                  >
                    <CardHeader className="pb-2">
                      <div className="flex gap-2">
                        <Tag
                          name={article.tag.name}
                          color={article.tag.color}
                        />
                        <div className="flex items-center text-xs text-gray-500 dark:text-neutral-400">
                          <Calendar className="mr-1 h-3 w-3" />
                          <time className="text-xs text-gray-500" dateTime={article.publishedDate}>
                            {format(new Date(article.publishedDate), 'MMM d, yyyy')}
                          </time>
                        </div>
                      </div>
                      <CardTitle className="text-lg text-gray-900 dark:text-white break-words whitespace-normal line-clamp-2 overflow-hidden leading-5">
                        {article.headline}
                      </CardTitle>
                    </CardHeader>
                    <CardContent className="flex gap-3">
                      {renderThumbnail(article)}
                      <div className="flex-1 min-w-0">
                        <CardDescription className="text-gray-600 dark:text-neutral-300 break-words whitespace-normal line-clamp-4">
                          {article.excerpt}
                        </CardDescription>
                      </div>
                    </CardContent>
                    <CardFooter>
                      <div className="flex items-center text-red-600 dark:text-red-400 mt-auto text-sm font-medium">
                        <BookOpen className="w-4 h-4 mr-1" />
                        <Link href={`/news/${article.slug}`}>
                          Read more
                        </Link>
                      </div>
                    </CardFooter>
                  </Card>
                ))
              )
            : (
                <Card className="w-[350px] flex-shrink-0">
                  <CardContent className="py-8">
                    <p className="text-center text-gray-500 dark:text-neutral-400">No news articles found</p>
                  </CardContent>
                </Card>
              )}
        </div>
        <ScrollBar orientation="horizontal" />
      </ScrollArea>
    </div>
  )
}

function renderThumbnail(article: PayloadNewsArticle) {
  if (article.featuredImage && article.featuredImage.url && article.featuredImage.url !== '') {
    return (
      <div className="relative w-20 h-16 overflow-hidden rounded bg-gray-200 dark:bg-neutral-700">
        <Image
          src={article.featuredImage.url}
          alt={article.featuredImage.alt}
          fill
          sizes="80px"
          className="object-cover"
          priority={false}
        />
      </div>
    )
  }
  else {
    return (
      <div className="w-20 h-16 flex items-center justify-center rounded bg-gray-200 dark:bg-neutral-700">
        <Newspaper className="w-8 h-8 text-gray-400 dark:text-neutral-500" />
      </div>
    )
  }
}
