'use client'
import type { CarouselApi } from '@/components/ui/carousel'
import type { Slide } from './types'
import {
  Carousel,
  CarouselContent,
  CarouselItem,
  CarouselNext,
  CarouselPrevious,
} from '@/components/ui/carousel'
import Image from 'next/image'
import React, { useEffect, useState } from 'react'

export default function Slideshow({ slides }: { slides: Slide[] }) {
  const [carouselApi, setCarouselApi] = useState<CarouselApi | null>(null)
  const [currentSlide, setCurrentSlide] = useState(1)
  const [isPaused, setIsPaused] = useState(false)

  useEffect(() => {
    if (!carouselApi)
      return

    const onSelect = (): void => {
      setCurrentSlide(carouselApi.selectedScrollSnap())
    }

    carouselApi.on('select', onSelect)

    onSelect()

    let intervalId: number | undefined

    if (!isPaused) {
      intervalId = window.setInterval(() => {
        carouselApi.scrollNext()
      }, 5000)
    }

    return () => {
      if (intervalId !== undefined) {
        clearInterval(intervalId)
      }
      carouselApi.off('select', onSelect)
    }
  }, [carouselApi, isPaused])

  const handleMouseEnter = (): void => setIsPaused(true)
  const handleMouseLeave = (): void => setIsPaused(false)

  return (
    <div
      className="w-full"
      onMouseEnter={handleMouseEnter}
      onMouseLeave={handleMouseLeave}
    >
      <div className="md:rounded-xl md:overflow-hidden md:mt-5">
        <Carousel
          setApi={setCarouselApi}
          className="w-full"
          opts={{
            loop: true,
            align: 'start',
          }}
        >
          <CarouselContent className="-ml-0">
            {slides.map(slide => (
              <CarouselItem key={slide.id} className="pl-0">
                <div className="relative w-full h-screen md:h-auto md:aspect-video overflow-hidden">
                  <div className="relative w-full h-full">
                    <Image
                      src={slide.url}
                      alt={slide.title || 'No title'}
                      fill
                      sizes="(max-width: 768px) 100vw, (max-width: 1200px) 100vw, 1200px"
                      className="object-cover"
                    />
                    <div className="absolute bottom-0 left-0 right-0 p-4 md:p-8 text-white z-20">
                      {/* Gradient above the text container */}
                      <div className="absolute -top-32 left-0 right-0 h-32 bg-gradient-to-b from-transparent to-black/40 z-10"></div>
                      {/* Black background behind text */}
                      <div className="absolute inset-0 bg-black/40 -z-10"></div>
                      <h2 className="text-xl md:text-3xl font-bold mb-1 md:mb-2">{slide.title}</h2>
                      <p className="text-sm md:text-lg mb-4 md:mb-0">{slide.description}</p>
                    </div>
                  </div>
                </div>
              </CarouselItem>
            ))}
          </CarouselContent>
          <CarouselPrevious className="left-4 bg-black/30 hover:bg-black/50 text-white border-none z-30" />
          <CarouselNext className="right-4 bg-black/30 hover:bg-black/50 text-white border-none z-30" />
        </Carousel>
      </div>

      <div className="flex justify-center gap-2 mt-2 md:mt-4 absolute md:static bottom-4 left-0 right-0 z-40">
        {slides.map((slide, index) => (
          <button
            key={slide.id}
            onClick={() => carouselApi?.scrollTo(index)}
            className={`w-2 h-2 md:w-3 md:h-3 rounded-full transition-colors ${
              index === currentSlide
                ? 'bg-white md:bg-primary'
                : 'bg-white/30 md:bg-primary/30'
            }`}
            aria-label="Go to selected slide"
            type="button"
          />
        ))}
      </div>
    </div>
  )
}
