import {
  DropdownMenu,
  DropdownMenuContent,
  DropdownMenuItem,
  DropdownMenuTrigger,
} from '@/components/ui/dropdown-menu'
import { Input } from '@/components/ui/input'
import config from '@/payload.config'
import { ChevronDown, Filter, Search } from 'lucide-react'
import Link from 'next/link'
import { getPayload } from 'payload'
import ExperimentSection from './ExperimentSection'

interface Experiment {
  id: string
  number: string
  title: string
  year: string
  speaker: {
    name: string
    link: string
  }
  groupleader: {
    name: string
    email: string
  }
  previewText: string
  featuredImage?: {
    url: string
    width: number
    height: number
    alt: string
  }
}

export default async function ExperimentPage({
  searchParams,
}: {
  searchParams: { sort?: string, q?: string }
}) {
  const payload = await getPayload({ config })

  const params = await searchParams
  const sortParam = params.sort || '-year'
  const searchQuery = params.q || ''

  const query: any = {
    collection: 'experiments',
    sort: sortParam,
    depth: 1,
  }

  // Add search functionality if query exists
  if (searchQuery) {
    query.where = {
      or: [
        { title: { contains: searchQuery } },
        { number: { contains: searchQuery } },
        { year: { contains: searchQuery } },
        // { speaker: { contains: searchQuery } },
        { previewText: { contains: searchQuery } },
      ],
    }
  }

  const result = await payload.find(query)

  const experiments = result.docs as unknown as Experiment[]

  // const featuredExperiments = articles.length > 0 ? articles[0] : null
  const remainingExperiments = experiments.length > 0 ? experiments : []

  const sortOptions = [
    { label: 'Newest first', value: '-year' },
    { label: 'Oldest first', value: 'year' },
    { label: 'A-Z', value: 'title' },
    { label: 'Z-A', value: '-title' },
  ]

  const currentSortOption = sortOptions.find(option => option.value === sortParam)?.label || 'Newest first'

  // const experiments = (await getPublications('experiments')).docs as unknown as Experiment[]

  return (
  // searchbar
    <div className="container mx-auto px-4 md:pt-5 pt-16">
      {/* Search and filters bar */}
      <div className="bg-white dark:bg-neutral-800 p-4 rounded-lg shadow-sm mb-8 border border-gray-200 dark:border-neutral-700">
        <form className="flex flex-col md:flex-row gap-4">
          <div className="relative flex-grow">
            <Search className="absolute left-3 top-1/2 transform -translate-y-1/2 h-4 w-4 text-gray-400" />
            <Input
              type="search"
              name="q"
              placeholder="Search experiments..."
              className="pl-10 bg-gray-50 dark:bg-neutral-900 border-gray-200 dark:border-neutral-700"
              defaultValue={searchQuery}
            />
          </div>
          <div className="flex gap-2">
            <DropdownMenu modal={false}>
              <DropdownMenuTrigger className="flex items-center gap-1 px-4 py-2 bg-gray-100 dark:bg-neutral-700 rounded-md border border-gray-200 dark:border-neutral-600 text-gray-700 dark:text-gray-200 outline-none">
                <Filter className="h-4 w-4" />
                <span className="text-sm hidden sm:inline">Sort by:</span>
                <span className="text-sm font-medium">{currentSortOption}</span>
                <ChevronDown className="h-4 w-4 ml-1" />
              </DropdownMenuTrigger>
              <DropdownMenuContent align="end">
                {sortOptions.map(option => (
                  <DropdownMenuItem key={option.value} asChild>
                    <Link href={`?sort=${option.value}${searchQuery ? `&q=${searchQuery}` : ''}`}>
                      {option.label}
                    </Link>
                  </DropdownMenuItem>
                ))}
              </DropdownMenuContent>
            </DropdownMenu>
            <button
              type="submit"
              className="bg-r3b-orange hover:bg-orange-700 text-white px-4 py-2 rounded-md transition-colors"
            >
              Search
            </button>
          </div>
        </form>
      </div>

      <div className="flex justify-between items-center mb-8">
        <h2 className="text-2xl md:text-3xl font-bold text-gray-800 dark:text-white">
          {searchQuery ? `Search Results: "${searchQuery}"` : 'Experiments'}
        </h2>
        <div className="text-sm text-gray-500 dark:text-gray-400">
          {experiments.length}
          {' '}
          experiment
          {experiments.length !== 1 ? 's' : ''}
          {' '}
          found
        </div>
      </div>

      {// every experiment that matches the search and sort parameters will be rendered here
      }
      <ExperimentSection experiments={remainingExperiments} />
    </div>
  )
}
