import { Card, CardContent } from '@/components/ui/card'
import {
  DropdownMenu,
  DropdownMenuContent,
  DropdownMenuItem,
  DropdownMenuTrigger,
} from '@/components/ui/dropdown-menu'
import { Input } from '@/components/ui/input'
import config from '@/payload.config'
import { format } from 'date-fns'
import { BookOpen, Calendar, ChevronDown, ChevronRight, Filter, Newspaper, Search, User } from 'lucide-react'
import Image from 'next/image'
import Link from 'next/link'
import { getPayload } from 'payload'
import Tag from '../components/Tag'

interface NewsArticle {
  id: string
  headline: string
  slug: string
  featuredImage?: {
    url: string
    width: number
    height: number
    alt: string
  }
  excerpt?: string
  publishedDate: string
  tag: {
    id: string
    name: string
    color: 'blue' | 'green' | 'red' | 'purple' | 'orange' | 'gray'
  }
  author: {
    id: string
    name: string
  }
}

export default async function NewsPage({
  searchParams,
}: {
  searchParams: { sort?: string, q?: string }
}) {
  const payload = await getPayload({ config })

  const params = await searchParams
  const sortParam = params.sort || '-publishedDate'
  const searchQuery = params.q || ''

  const query: any = {
    collection: 'news-articles',
    sort: sortParam,
    depth: 1,
  }

  // Add search functionality if query exists
  if (searchQuery) {
    query.where = {
      or: [
        { headline: { contains: searchQuery } },
        { excerpt: { contains: searchQuery } },
      ],
    }
  }

  const result = await payload.find(query)

  const articles = result.docs as unknown as NewsArticle[]

  const featuredArticle = articles.length > 0 ? articles[0] : null
  const remainingArticles = articles.length > 0 ? articles : []

  const sortOptions = [
    { label: 'Newest first', value: '-publishedDate' },
    { label: 'Oldest first', value: 'publishedDate' },
    { label: 'A-Z', value: 'headline' },
    { label: 'Z-A', value: '-headline' },
  ]

  const currentSortOption = sortOptions.find(option => option.value === sortParam)?.label || 'Newest first'

  return (
    <main className="bg-gray-50 dark:bg-neutral-950 min-h-screen pb-16 md:pt-5">
      {/* Hero section with featured article */}
      {featuredArticle && (
        <div className="relative w-full h-96 md:h-[500px] overflow-hidden mb-12 md:rounded-xl">
          <div className="absolute inset-0 bg-gradient-to-b from-black/60 via-black/40 to-black/70 z-10" />

          {featuredArticle.featuredImage && featuredArticle.featuredImage.url
            ? (
                <Image
                  src={featuredArticle.featuredImage.url}
                  alt={featuredArticle.featuredImage.alt || featuredArticle.headline}
                  fill
                  className="object-cover"
                  priority
                />
              )
            : (
                <div className="absolute inset-0 bg-gray-800 flex items-center justify-center">
                  <Newspaper className="w-32 h-32 text-gray-600" />
                </div>
              )}

          <div className="container mx-auto px-4 relative z-20 h-full flex flex-col justify-end pb-12">
            <div className="max-w-3xl">
              <div className="flex gap-3 mb-3">
                <Tag
                  name={featuredArticle.tag.name}
                  color={featuredArticle.tag.color}
                  className="text-sm"
                />
                <div className="flex items-center bg-black/30 text-white rounded-full px-3 py-1 text-xs">
                  <Calendar className="w-3 h-3 mr-1" />
                  <time dateTime={featuredArticle.publishedDate}>
                    {format(new Date(featuredArticle.publishedDate), 'MMM d, yyyy')}
                  </time>
                </div>
              </div>

              <h1 className="text-3xl md:text-4xl lg:text-5xl font-bold text-white mb-4">
                {featuredArticle.headline}
              </h1>

              {featuredArticle.excerpt && (
                <p className="text-white/90 text-lg md:text-xl mb-4 line-clamp-2">
                  {featuredArticle.excerpt}
                </p>
              )}

              <div className="flex items-center justify-between">
                <div className="flex items-center text-white/80">
                  <User className="w-4 h-4 mr-2" />
                  <span>
                    By
                    {featuredArticle.author.name}
                  </span>
                </div>

                <Link
                  href={`/news/${featuredArticle.slug}`}
                  className="flex items-center bg-r3b-orange hover:bg-orange-700 text-white px-4 py-2 rounded-md transition-colors"
                >
                  <span>Read article</span>
                  <ChevronRight className="w-4 h-4 ml-1" />
                </Link>
              </div>
            </div>
          </div>
        </div>
      )}

      <div className="container mx-auto px-4">
        {/* Search and filters bar */}
        <div className="bg-white dark:bg-neutral-800 p-4 rounded-lg shadow-sm mb-8 border border-gray-200 dark:border-neutral-700">
          <form className="flex flex-col md:flex-row gap-4">
            <div className="relative flex-grow">
              <Search className="absolute left-3 top-1/2 transform -translate-y-1/2 h-4 w-4 text-gray-400" />
              <Input
                type="search"
                name="q"
                placeholder="Search news articles..."
                className="pl-10 bg-gray-50 dark:bg-neutral-900 border-gray-200 dark:border-neutral-700"
                defaultValue={searchQuery}
              />
            </div>
            <div className="flex gap-2">
              <DropdownMenu modal={false}>
                <DropdownMenuTrigger className="flex items-center gap-1 px-4 py-2 bg-gray-100 dark:bg-neutral-700 rounded-md border border-gray-200 dark:border-neutral-600 text-gray-700 dark:text-gray-200 outline-none">
                  <Filter className="h-4 w-4" />
                  <span className="text-sm hidden sm:inline">Sort by:</span>
                  <span className="text-sm font-medium">{currentSortOption}</span>
                  <ChevronDown className="h-4 w-4 ml-1" />
                </DropdownMenuTrigger>
                <DropdownMenuContent align="end">
                  {sortOptions.map(option => (
                    <DropdownMenuItem key={option.value} asChild>
                      <Link href={`?sort=${option.value}${searchQuery ? `&q=${searchQuery}` : ''}`}>
                        {option.label}
                      </Link>
                    </DropdownMenuItem>
                  ))}
                </DropdownMenuContent>
              </DropdownMenu>
              <button
                type="submit"
                className="bg-r3b-orange hover:bg-orange-700 text-white px-4 py-2 rounded-md transition-colors"
              >
                Search
              </button>
            </div>
          </form>
        </div>

        <div className="flex justify-between items-center mb-8">
          <h2 className="text-2xl md:text-3xl font-bold text-gray-800 dark:text-white">
            {searchQuery ? `Search Results: "${searchQuery}"` : 'Latest News'}
          </h2>
          <div className="text-sm text-gray-500 dark:text-gray-400">
            {articles.length}
            {' '}
            article
            {articles.length !== 1 ? 's' : ''}
            {' '}
            found
          </div>
        </div>

        {/* News articles list */}
        {remainingArticles.length > 0
          ? (
              <div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 gap-8">
                {remainingArticles.map(article => (
                  <Link
                    key={article.id}
                    href={`/news/${article.slug}`}
                    className="group"
                  >
                    <Card className="overflow-hidden h-full flex flex-col bg-white dark:bg-neutral-800 border border-gray-200 dark:border-neutral-700 group-hover:border-red-300 dark:group-hover:border-red-800 transition-colors shadow-sm group-hover:shadow-md">
                      <div className="relative w-full h-48 overflow-hidden bg-gray-200 dark:bg-neutral-700">
                        {article.featuredImage && article.featuredImage.url
                          ? (
                              <Image
                                src={article.featuredImage.url}
                                alt={article.featuredImage.alt || article.headline}
                                fill
                                className="object-cover transition-transform group-hover:scale-105"
                              />
                            )
                          : (
                              <div className="w-full h-full flex items-center justify-center">
                                <Newspaper className="w-12 h-12 text-gray-400 dark:text-neutral-500" />
                              </div>
                            )}

                        <div className="absolute top-3 left-3">
                          <Tag
                            name={article.tag.name}
                            color={article.tag.color}
                          />
                        </div>
                      </div>

                      <CardContent className="flex-grow flex flex-col p-5">
                        <div className="flex items-center text-xs text-gray-500 dark:text-gray-400 mb-2">
                          <Calendar className="w-3 h-3 mr-1" />
                          <time dateTime={article.publishedDate}>
                            {format(new Date(article.publishedDate), 'MMM d, yyyy')}
                          </time>
                          <span className="mx-2">•</span>
                          <User className="w-3 h-3 mr-1" />
                          <span>{article.author.name}</span>
                        </div>

                        <h3 className="text-xl font-semibold text-gray-900 dark:text-white mb-3 line-clamp-2 group-hover:text-red-600 dark:group-hover:text-red-400 transition-colors">
                          {article.headline}
                        </h3>

                        {article.excerpt && (
                          <p className="text-gray-600 dark:text-gray-300 mb-4 line-clamp-3 flex-grow">
                            {article.excerpt}
                          </p>
                        )}

                        <div className="flex items-center text-red-600 dark:text-red-400 mt-auto pt-2 text-sm font-medium">
                          <BookOpen className="w-4 h-4 mr-1" />
                          <span>Read more</span>
                        </div>
                      </CardContent>
                    </Card>
                  </Link>
                ))}
              </div>
            )
          : (
              <Card className="p-12 text-center bg-white dark:bg-neutral-800">
                <p className="text-gray-500 dark:text-gray-400">No news articles found</p>
              </Card>
            )}
      </div>
    </main>
  )
}
