import type { Publication, PublicationFetchOptions, PublicationListResult } from '@/lib/publications'
import {
  Card,
  CardContent,
  CardDescription,
  CardFooter,
  CardHeader,
  CardTitle,
} from '@/components/ui/card'
import {
  DropdownMenu,
  DropdownMenuContent,
  DropdownMenuItem,
  DropdownMenuTrigger,
} from '@/components/ui/dropdown-menu'

import { Input } from '@/components/ui/input'
import { Separator } from '@/components/ui/separator'
import { Skeleton } from '@/components/ui/skeleton'
import { fetchPublications } from '@/lib/publications'
import { ChevronDown, Filter, Search } from 'lucide-react'
import Link from 'next/link'
import { Suspense } from 'react'
import PaginationUI from '../components/PaginationUI'

const POSTS_PER_PAGE = 15

async function PublicationCard({ publication }: { publication: Publication }) {
  return (
    <Card className="overflow-hidden transition-all hover:shadow-md border border-gray-200 dark:border-neutral-800 hover:border-orange-400 dark:hover:border-r3b-orange">
      <CardHeader className="bg-stone-100 dark:bg-stone-900 pb-2">
        <CardTitle className="text-lg font-bold line-clamp-2">
          {publication.title}
        </CardTitle>
        <CardDescription className="line-clamp-1">
          {publication.authors}
        </CardDescription>
      </CardHeader>
      <CardContent className="pt-4 pb-2">
        <div className="space-y-2">
          <div className="flex items-start gap-1 text-sm">
            <span className="font-medium text-slate-700 dark:text-slate-300 min-w-20">Journal:</span>
            <span>{publication.journal}</span>
          </div>
          <div className="flex items-start gap-1 text-sm">
            <span className="font-medium text-slate-700 dark:text-slate-300 min-w-20">Published:</span>
            <span>{publication.published}</span>
            {publication.volume && (
              <span>
                • Vol
                {publication.volume}
              </span>
            )}
            {publication.pages && (
              <span>
                •
                {publication.pages}
                {' '}
                pages
              </span>
            )}
          </div>
          <div className="flex items-start gap-1 text-sm">
            <span className="font-medium text-slate-700 dark:text-slate-300 min-w-20">DOI:</span>
            <span className="text-orange-600 dark:text-orange-400">{publication.doi}</span>
          </div>
        </div>
      </CardContent>
      <CardFooter className="pt-0">
        <Link
          href={`publications/${publication.id}`}
          rel="noopener noreferrer"
          className="text-sm text-orange-600 dark:text-orange-400 hover:underline"
        >
          View Publication →
        </Link>
      </CardFooter>
    </Card>
  )
}

function PublicationsSkeleton() {
  return (
    <div className="grid gap-4 md:grid-cols-2 lg:grid-cols-3">
      {[...Array.from({ length: 6 })].map((_, index) => (
        // eslint-disable-next-line react/no-array-index-key
        <Card key={index} className="overflow-hidden">
          <CardHeader className="pb-2">
            <Skeleton className="h-6 w-full" />
            <Skeleton className="h-4 w-3/4 mt-2" />
          </CardHeader>
          <CardContent className="pt-4 pb-2">
            <div className="space-y-2">
              <Skeleton className="h-4 w-full" />
              <Skeleton className="h-4 w-full" />
              <Skeleton className="h-4 w-3/4" />
            </div>
          </CardContent>
          <CardFooter>
            <Skeleton className="h-4 w-1/3" />
          </CardFooter>
        </Card>
      ))}
    </div>
  )
}

export default async function PublicationsPage({
  searchParams,
}: {
  searchParams: { sort?: string, q?: string, page?: string }
}) {
  // Needs to be awaited in order to suppress warning in browser console
  const params = await searchParams
  const sortParam = params?.sort || ''
  const searchQuery = params.q || ''
  const currentPage = Number(params?.page) || 1

  const options: PublicationFetchOptions = {
    page: currentPage,
    limit: POSTS_PER_PAGE,
    depth: 1,
    sort: sortParam,
  }

  // Add search functionality if query exists
  if (searchQuery) {
    options.where = {
      or: [
        { title: { contains: searchQuery } },
        { authors: { contains: searchQuery } },
        { journal: { contains: searchQuery } },
        { doi: { contains: searchQuery } },
      ],
    }
  }

  const publications: PublicationListResult = await fetchPublications(options)

  const sortOptions = [
    { label: 'Newest first', value: '-published' },
    { label: 'Oldest first', value: 'published' },
    { label: 'A-Z', value: 'title' },
    { label: 'Z-A', value: '-title' },
  ]
  const currentSortOption = sortOptions.find(option => option.value === sortParam)?.label || 'Newest first'

  return (
    <div className="container max-w-7xl mx-auto py-8 px-4">
      <header className="mb-8 mt-8 md:mt-0">
        <h1 className="text-3xl font-bold mb-2">Scientific Publications</h1>
        <p className="text-slate-600 dark:text-slate-400 mb-6">
          Browse our collection of
          {' '}
          {publications.totalDocs}
          {' '}
          scientific publications.
        </p>

        <div className="bg-white dark:bg-neutral-900 p-4 rounded-lg shadow-sm mb-8 border border-gray-200 dark:border-neutral-800">
          <form className="flex flex-col md:flex-row gap-4">
            <div className="relative flex-grow">
              <Search className="absolute left-3 top-1/2 transform -translate-y-1/2 h-4 w-4 text-gray-400" />
              <Input
                type="search"
                name="q"
                placeholder="Search publications..."
                className="pl-10 bg-gray-50 dark:bg-neutral-950 border-gray-200 dark:border-neutral-800"
                defaultValue={searchQuery}
              />
            </div>
            <div className="flex gap-2">
              <DropdownMenu modal={false}>
                <DropdownMenuTrigger className="flex items-center gap-1 px-4 py-2 bg-gray-100 dark:bg-neutral-800 rounded-md border border-gray-200 dark:border-neutral-700 text-gray-700 dark:text-gray-200 outline-none">
                  <Filter className="h-4 w-4" />
                  <span className="text-sm hidden sm:inline">Sort by:</span>
                  <span className="text-sm font-medium">{currentSortOption}</span>
                  <ChevronDown className="h-4 w-4 ml-1" />
                </DropdownMenuTrigger>
                <DropdownMenuContent align="end">
                  {sortOptions.map(option => (
                    <DropdownMenuItem key={option.value} asChild>
                      <Link href={`?sort=${option.value}${searchQuery ? `&q=${searchQuery}` : ''}`}>
                        {option.label}
                      </Link>
                    </DropdownMenuItem>
                  ))}
                </DropdownMenuContent>
              </DropdownMenu>
              <button
                type="submit"
                className="bg-r3b-orange hover:bg-orange-700 text-white px-4 py-2 rounded-md transition-colors"
              >
                Search
              </button>
            </div>
          </form>
        </div>
      </header>

      <Separator className="my-6" />

      <main>
        <Suspense fallback={<PublicationsSkeleton />}>
          <div className="grid gap-6 sm:grid-cols-1 md:grid-cols-2 lg:grid-cols-3 mb-6">
            {publications.docs.map((publication: Publication) => (
              <PublicationCard key={publication.id} publication={publication} />
            ))}
          </div>
        </Suspense>

        <PaginationUI
          currentPage={currentPage}
          totalPages={publications.totalPages}
          totalDocs={publications.totalDocs}
        />
      </main>
    </div>
  )
}
