import type { CollectionConfig } from 'payload'
import { lexicalEditor } from '@payloadcms/richtext-lexical'

interface LexicalNode {
  type: string
  text?: string
  children?: LexicalNode[]
}

interface LexicalContent {
  root?: {
    children?: LexicalNode[]
  }
}

interface BeforeValidateArgs {
  value?: string | null
  siblingData: {
    content?: LexicalContent
    excerpt?: string
  }
  originalDoc?: {
    content?: LexicalContent
    excerpt?: string
  }
}
export const NewsArticles: CollectionConfig = {
  slug: 'news-articles',
  labels: {
    singular: 'News Article',
    plural: 'News Articles',
  },
  access: {
    read: () => true,
  },
  versions: {
    drafts: true,
  },
  fields: [
    {
      name: 'headline',
      type: 'text',
      required: true,
      label: 'Headline',
      admin: {
        description: 'The main headline of the article',
      },
    },
    {
      name: 'subheadline',
      type: 'text',
      required: false,
      label: 'Subheadline',
      admin: {
        description: 'Optional secondary headline',
      },
    },
    {
      name: 'slug',
      type: 'text',
      required: true,
      unique: true,
      admin: {
        description: 'URL-friendly version of the headline (auto-generated)',
        position: 'sidebar',
      },
      hooks: {
        beforeValidate: [
          ({ value, siblingData }) => {
            if (!value && siblingData?.headline) {
              return siblingData.headline
                .toLowerCase()
                .trim()
                .replace(/[^\w\s]/g, '')
                .replace(/\s+/g, '-')
            }
            return value
          },
        ],
      },
    },
    {
      name: 'featuredImage',
      type: 'upload',
      relationTo: 'news-article-images',
      label: 'Featured Image',
      admin: {
        description: 'The main image for the article',
      },
    },
    {
      name: 'content',
      type: 'richText',
      label: 'Article Content',
      required: true,
      editor: lexicalEditor({
        features: ({ defaultFeatures, rootFeatures }) => [
          ...defaultFeatures,
          ...rootFeatures,
        ],
      }),
    },
    {
      name: 'excerpt',
      type: 'textarea',
      admin: {
        description: 'Brief summary of the article (will be auto-generated if left empty)',
      },
      hooks: {
        beforeValidate: [
          ({ value, siblingData, originalDoc }: BeforeValidateArgs): string | null => {
            const contentChanged = originalDoc
              && JSON.stringify(originalDoc.content) !== JSON.stringify(siblingData?.content)

            const shouldGenerateExcerpt = !value
              || (contentChanged && originalDoc?.excerpt === value)

            if (!shouldGenerateExcerpt || !siblingData?.content) {
              return value || null
            }

            const extractTextFromNodes = (nodes?: LexicalNode[]): string => {
              if (!Array.isArray(nodes))
                return ''

              return nodes.reduce((text, node) => {
                if (node.type === 'text' && node.text) {
                  return text + node.text
                }

                if (node.children) {
                  return `${text} ${extractTextFromNodes(node.children)}`
                }

                return text
              }, '')
            }

            const textContent = (() => {
              try {
                const rootChildren = siblingData.content.root?.children
                return rootChildren ? extractTextFromNodes(rootChildren).trim() : ''
              }
              catch {
                return ''
              }
            })()

            return textContent.length > 200
              ? `${textContent.substring(0, 200)}...`
              : textContent
          },
        ],
      },
    },
    {
      name: 'tag',
      type: 'relationship',
      relationTo: 'tags',
      hasMany: false,
      required: true,
      admin: {
        description: 'Select or create tags for this article',
        position: 'sidebar',
      },
    },
    {
      name: 'author',
      type: 'relationship',
      relationTo: 'users',
      hasMany: false,
      required: true,
      admin: {
        position: 'sidebar',
        description: 'Select the author of this article',
      },
    },
    {
      name: 'publishedDate',
      type: 'date',
      required: true,
      admin: {
        position: 'sidebar',
        date: {
          pickerAppearance: 'dayAndTime',
        },
      },
    },
  ],
}
