import type { CollectionConfig } from 'payload'
import { generateBibTeX } from '@/lib/bibtexGenerator'

interface InspireResponse {
  metadata: {
    titles: Array<{ title: string, source: string }>
    authors: Array<{ full_name: string }>
    publication_info: Array<{
      journal_title: string
      journal_volume: string
      year: string
      page_start: string
      page_end: string
      artid: string
    }>
    abstracts: Array<{ value: string }>
    number_of_pages: string
    dois: Array<{ value: string }>
  }
}

interface PublicationData {
  title: string
  source: string
  authors: string
  journal: string
  volume: string
  published: string
  pages: string
  abstract: string
  doi: string
  link: string
}

export const Publications: CollectionConfig = {
  slug: 'publications',
  admin: {
    useAsTitle: 'title',
    defaultColumns: ['title', 'authors', 'journal', 'published'],
  },
  fields: [
    {
      name: 'doi',
      label: 'DOI',
      type: 'text',
      required: true,
      hooks: {
        beforeValidate: [
          ({ value }) => {
            return value.trim()
          },
        ],
      },
    },
    {
      name: 'title',
      label: 'Title',
      type: 'text',
      required: false,
    },
    {
      name: 'source',
      label: 'Source',
      type: 'text',
      required: false,
    },
    {
      name: 'authors',
      label: 'Authors',
      type: 'text',
      required: false,
    },
    {
      name: 'journal',
      label: 'Journal',
      type: 'text',
      required: false,
    },
    {
      name: 'volume',
      label: 'Volume',
      type: 'text',
      required: false,
    },
    {
      name: 'published',
      label: 'Year Published',
      type: 'text',
      required: false,
    },
    {
      name: 'pages',
      label: 'Pages',
      type: 'text',
      required: false,
    },
    {
      name: 'abstract',
      label: 'Abstract',
      type: 'textarea',
      required: false,
    },
    {
      name: 'link',
      label: 'Publisher Link',
      type: 'text',
      required: false,
    },
    {
      name: 'pdfFile',
      label: 'PDF File',
      type: 'upload',
      relationTo: 'publication-documents',
      required: false,
    },
    {
      name: 'relatedPublications',
      label: 'Related Publications',
      type: 'relationship',
      relationTo: 'publications',
      hasMany: true,
      required: false,
    },
    {
      name: 'tags',
      label: 'Tags',
      type: 'array',
      fields: [
        {
          name: 'tag',
          type: 'text',
        },
      ],
    },
  ],
  hooks: {
    beforeChange: [
      async ({ data, originalDoc, operation }) => {
        if (operation === 'create' || originalDoc?.doi !== data.doi) {
          try {
            const publication = await getPublicationByDOI(data.doi)
            if (publication) {
              Object.assign(data, publication)
            }
          }
          catch (error) {
            console.error('Failed to fetch publication: ', error)
            throw new Error('Failed to fetch publication data')
          }
        }
        return data
      },
    ],
  },
}

/**
 * fetches publication from INSPIRE REST API: https://github.com/inspirehep/rest-api-doc
 *
 * @param doi of the publication
 * @returns publication data or null if not found
 */
async function getPublicationByDOI(doi: string): Promise<PublicationData | null> {
  try {
    const response = await fetch(`https://inspirehep.net/api/doi/${doi}`)
    if (!response.ok) {
      throw new Error(`API request failed with status ${response.status}`)
    }

    const data: InspireResponse = await response.json()
    const pubInfo = data.metadata.publication_info[0] || {}

    // Format pages (can be either page_start-page_end or artid)
    let formattedPages = ''
    if (pubInfo.page_start && pubInfo.page_end) {
      formattedPages = `${pubInfo.page_start}-${pubInfo.page_end}`
    }
    else if (pubInfo.page_start) {
      formattedPages = pubInfo.page_start
    }
    else if (pubInfo.artid) {
      formattedPages = pubInfo.artid
    }
    else {
      formattedPages = data.metadata.number_of_pages ? `${data.metadata.number_of_pages} pp.` : ''
    }

    return {
      title: data.metadata.titles[0]?.title ?? '',
      source: data.metadata.titles[0]?.source ?? '',
      authors: formatAuthors(data.metadata.authors),
      journal: pubInfo.journal_title ?? 'independant',
      volume: pubInfo.journal_volume ?? '',
      published: pubInfo.year ?? '',
      pages: formattedPages,
      abstract: data.metadata.abstracts?.[0]?.value ?? '',
      doi: data.metadata.dois?.[0]?.value ?? doi,
      link: `https://doi.org/${doi}`,
    }
  }
  catch (error) {
    console.error('Error fetching publication: ', error)
    return null
  }
}

/**
 * Formats authors from the API response into a readable string
 *
 * @param authorsUnformated gets an array of objects with element full_name
 * @returns formated names as a string. e.g. ["John, Wick"], ["Harry, Styles"] -> "John Wick, Harry Styles"
 */
function formatAuthors(authorsUnformated: { full_name: string }[]): string {
  if (!authorsUnformated || !Array.isArray(authorsUnformated)) {
    return ''
  }

  const authors: string[] = []

  authorsUnformated.forEach((author: { full_name: string }) => {
    authors.push(author.full_name.replace(',', ''))
  })

  return authors.join(', ')
}

export default Publications
