/**
 * Generates BibTeX citations from Publication objects
 */

import type { Publication } from './publications'

/**
 * Generates a BibTeX citation key from author and year
 * Format: LastName:YearXXX where XXX are random letters
 */
function generateCitationKey(authors: string, published: string): string {
  // Extract the first author's last name
  const firstAuthor = authors.split(',')[0].trim()
  const lastName = firstAuthor.split(' ').pop() || 'Unknown'

  // Extract the year from the published date
  const year = new Date(published).getFullYear()

  // Generate three random lowercase letters
  const randomLetters = Math.random().toString(36).substring(2, 5)

  return `${lastName}:${year}${randomLetters}`
}

/**
 * Formats author names for BibTeX
 * Input: "John Doe, Jane Smith"
 * Output: "Doe, John and Smith, Jane"
 */
function formatAuthors(authors: string): string {
  return authors
    .split(',')
    .map((author) => {
      const names = author.trim().split(' ')
      const lastName = names.pop() || ''
      const firstName = names.join(' ')
      return `${lastName}, ${firstName}`.trim()
    })
    .join(' and ')
}

/**
 * Extract primary class (category) from arXiv ID if available
 */
function extractPrimaryClass(link: string): string | null {
  // Check if it's an arXiv link
  const arXivMatch = link.match(/arxiv\.org\/abs\/(\d+\.\d+)/i)
  if (!arXivMatch)
    return null

  // For newer arXiv papers, we assume hep-th as default
  // In a real application, you would fetch this information
  return 'hep-th'
}

/**
 * Extract arXiv ID from link
 */
function extractArXivId(link: string): string | null {
  const match = link.match(/arxiv\.org\/abs\/(\d+\.\d+)/i)
  return match ? match[1] : null
}

/**
 * Format a single publication as BibTeX
 */
export function generateBibTeX(pub: Publication | null): string {
  // Generate citation key
  if (!pub)
    return ''
  const citationKey = generateCitationKey(pub.authors, pub.published)

  // Format authors
  const formattedAuthors = formatAuthors(pub.authors)

  // Extract year (and possibly month) from published date
  const pubDate = new Date(pub.published)
  const year = pubDate.getFullYear()

  // Determine entry type (article, book, etc.)
  const entryType = pub.journal ? 'article' : 'misc'

  // Start building BibTeX entry
  let bibtex = `@${entryType}{${citationKey},\n`
  bibtex += `  author = "${formattedAuthors}",\n`
  bibtex += `  title = "{${pub.title}}",\n`

  // Add arXiv information if available
  const arXivId = extractArXivId(pub.link)
  if (arXivId) {
    bibtex += `  eprint = "${arXivId}",\n`
    bibtex += `  archivePrefix = "arXiv",\n`

    const primaryClass = extractPrimaryClass(pub.link)
    if (primaryClass) {
      bibtex += `  primaryClass = "${primaryClass}",\n`
    }
  }

  // Add DOI if available
  if (pub.doi) {
    bibtex += `  doi = "${pub.doi}",\n`
  }

  // Add journal information if available
  if (pub.journal) {
    bibtex += `  journal = "${pub.journal}",\n`

    if (pub.volume) {
      bibtex += `  volume = "${pub.volume}",\n`
    }

    // Number is not in your interface, but would go here
    // if (pub.number) {
    //   bibtex += `  number = "${pub.number}",\n`;
    // }

    if (pub.pages) {
      bibtex += `  pages = "${pub.pages}",\n`
    }
  }

  // Add year (required)
  bibtex += `  year = "${year}"\n`

  // Close the entry
  bibtex += `}`

  return bibtex
}
