import type { NewsArticle } from '@/payload-types'

const debounceMap = new Map<string, NodeJS.Timeout>()

/**
 * Calls the callback function after delay. If a function is called via an id that is already being delayed, the delay timer is being restarted.
 * @param callback function to be debounced
 * @param id to identify which timer to delete
 * @param delay the amount of time to delay the function call
 */
export function debounce<T extends unknown[]>(callback: (...args: T) => void, id: string, delay: number = 500) {
  ((...args: T) => {
    if (debounceMap.has(id))
      clearTimeout(debounceMap.get(id))
    const timeout = setTimeout(() => {
      debounceMap.delete(id)
      callback(...args)
    }, delay)
    debounceMap.set(id, timeout)
  }).apply(globalThis)
}

/**
 *
 * @param article the newsArticle
 * @returns an array containing the content of the relevant fields for the searchquery
 */
export function getArticleAsArray(article: NewsArticle) {
  const articleArray = [article.createdAt, article.expireDate, article.infotext, article.slug, article.title, article.Article.articleHeadline, article.Article.articlePreviewText, article.Article.subHeadline]
  const blocksArray = article.Article.layout.flatMap((block) => {
    if (block.blockType === 'image') {
      return [block.headline]
    }
    else if (block.blockType === 'imageParagraph') {
      return [block.headline, block.text]
    }
    else if (block.blockType === 'paragraph') {
      return [block.headline, block.text]
    }
    else {
      return ''
    }
  })
  return articleArray.concat(blocksArray)
}
