import config from '@payload-config'
import { getPayload } from 'payload'

export interface Publication {
  id: string
  title: string
  doi: string
  authors: string
  journal: string
  volume: string
  published: string
  pages: string
  abstract: string
  link: string
  source?: string
  pdfFile?: {
    id: string
    filename: string
    url: string
  }
  relatedPublications?: Publication[]
  tags?: { tag: string }[]
  createdAt: string
  updatedAt: string
}

export interface PublicationListResult {
  docs: Publication[]
  totalDocs: number
  limit: number
  totalPages: number
  page: number
  pagingCounter: number
  hasPrevPage: boolean
  hasNextPage: boolean
  prevPage: number | null
  nextPage: number | null
}

export interface PublicationFetchOptions {
  page?: number
  limit?: number
  sort?: string
  where?: Record<string, any>
  depth?: number
}

/**
 * Initialize the payload instance
 * @returns Initialized payload instance
 */
export async function getPayloadInstance() {
  return await getPayload({ config })
}

/**
 * Fetch all publications with pagination
 * @param options Query options including pagination, sorting, filtering
 * @returns Paginated publication results
 */
export async function fetchPublications(options: PublicationFetchOptions = {}): Promise<PublicationListResult> {
  const payload = await getPayloadInstance()

  const {
    page = 1,
    limit = 10,
    sort = '-createdAt',
    where = {},
    depth = 1,
  } = options

  return await payload.find({
    collection: 'publications',
    page,
    limit,
    sort,
    where,
    depth,
  })
}

/**
 * Fetch a single publication by its ID
 * @param id Publication ID
 * @param depth Depth of relationship population
 * @returns Publication or null if not found
 */
export async function fetchPublicationById(id: string, depth = 1): Promise<Publication | null> {
  const payload = await getPayloadInstance()

  try {
    return await payload.findByID({
      collection: 'publications',
      id,
      depth,
      disableErrors: true,
    })
  }
  catch (error) {
    console.error(`Error fetching publication by ID ${id}:`, error)
    return null
  }
}

/**
 * Fetch a publication by DOI
 * @param doi Publication DOI
 * @returns Publication or null if not found
 */
export async function fetchPublicationByDOI(doi: string): Promise<Publication | null> {
  const payload = await getPayloadInstance()

  try {
    const result = await payload.find({
      collection: 'publications',
      where: {
        doi: {
          equals: doi,
        },
      },
      limit: 1,
    })

    return result.docs.length > 0 ? result.docs[0] : null
  }
  catch (error) {
    console.error(`Error fetching publication by DOI ${doi}:`, error)
    return null
  }
}

/**
 * Search publications by title, authors, or journal
 * @param query Search query
 * @param options Additional query options
 * @returns Paginated publication results
 */
export async function searchPublications(
  query: string,
  options: PublicationFetchOptions = {},
): Promise<PublicationListResult> {
  const searchWhere = {
    or: [
      {
        title: {
          contains: query,
        },
      },
      {
        authors: {
          contains: query,
        },
      },
      {
        journal: {
          contains: query,
        },
      },
      {
        abstract: {
          contains: query,
        },
      },
      {
        doi: {
          contains: query,
        },
      },
    ],
  }

  const mergedWhere = options.where
    ? { and: [searchWhere, options.where] }
    : searchWhere

  return await fetchPublications({
    ...options,
    where: mergedWhere,
  })
}

/**
 * Fetch publications by tag
 * @param tag Tag to filter by
 * @param options Additional query options
 * @returns Paginated publication results
 */
export async function fetchPublicationsByTag(
  tag: string,
  options: PublicationFetchOptions = {},
): Promise<PublicationListResult> {
  const tagWhere = {
    'tags.tag': {
      contains: tag,
    },
  }

  const mergedWhere = options.where
    ? { and: [tagWhere, options.where] }
    : tagWhere

  return await fetchPublications({
    ...options,
    where: mergedWhere,
  })
}

/**
 * Fetch publications by year
 * @param year Year published
 * @param options Additional query options
 * @returns Paginated publication results
 */
export async function fetchPublicationsByYear(
  year: string,
  options: PublicationFetchOptions = {},
): Promise<PublicationListResult> {
  const yearWhere = {
    published: {
      equals: year,
    },
  }

  const mergedWhere = options.where
    ? { and: [yearWhere, options.where] }
    : yearWhere

  return await fetchPublications({
    ...options,
    where: mergedWhere,
  })
}

/**
 * Count total publications matching a query
 * @param where Where conditions
 * @returns Count of matching publications
 */
export async function countPublications(where: Record<string, any> = {}): Promise<number> {
  const payload = await getPayloadInstance()

  try {
    const result = await payload.count({
      collection: 'publications',
      where,
    })

    return result.totalDocs
  }
  catch (error) {
    console.error('Error counting publications:', error)
    return 0
  }
}
